# http://flask.pocoo.org/snippets/62/
import base64
from os import urandom
from urllib.parse import urlparse, urljoin
from flask import request
from itsdangerous import URLSafeTimedSerializer, BadSignature
from projects import app
from projects import login_manager
from projects.models import User


def is_safe_url(target):
    ref_url = urlparse(request.host_url)
    test_url = urlparse(urljoin(request.host_url, target))
    return test_url.scheme in ("http", "https") and ref_url.netloc == test_url.netloc


# LoginManager
@login_manager.user_loader
def load_user(user_id):
    user = User.get(user_id)
    return user


def generate_confirmation_token(data):
    serializer = URLSafeTimedSerializer(app.config["SECRET_KEY"])
    return serializer.dumps(data, salt=app.config["ONETIME_TOKEN_SALT"])


# A token is valid for a day
def confirm_token(token, expiration=86400):
    serializer = URLSafeTimedSerializer(app.config["SECRET_KEY"])
    try:
        data = serializer.loads(
            token, salt=app.config["ONETIME_TOKEN_SALT"], max_age=expiration
        )
    except BadSignature:
        return False
    return data


def unique_name_encoding(name):
    """
    :param name: raw string name
    :return: the raw string name is padded with a 10 byte urandom string
    extension
    """
    assert type(name) == str
    return name + "_" + str(base64.b64encode(urandom(10)))


def unique_name_decode(name):
    """
    :param name: encoded name
    :return:
    """
    assert type(name) == str
    idx = name.rfind("_")
    return name[:idx]



from flask_wtf import FlaskForm
from flask_wtf.file import FileRequired, FileAllowed
from wtforms import (
    StringField,
    TextAreaField,
    FileField,
    SelectMultipleField,
    widgets,
    PasswordField,
)
from wtforms.validators import DataRequired, Regexp, Email, EqualTo, Length
from projects.conf import config


class MultiCheckboxField(SelectMultipleField):
    widget = widgets.ListWidget(prefix_label=False)
    option_widget = widgets.CheckboxInput()


class ProjectFormManager:
    form_class = None

    def register_form_class(self, class_name, module=None):
        # Look in external module if provided, else try in the
        # projects module
        if module:
            module_ = __import__(module)
            class_ = getattr(module_, class_name)
        else:
            module_ = __import__("projects")
            class_ = getattr(module_, class_name)
        self.form_class = class_

    def get_form_class(self):
        if self.form_class is None:
            self.register_form_class(
                config.get("PROJECTS", "form_class"),
                config.get("PROJECTS", "form_module", **{"fallback": None}),
            )
        return self.form_class


class DefaultProjectForm(FlaskForm):
    name = StringField("Name", validators=[DataRequired()])
    description = TextAreaField("Description")
    image = FileField(
        "Project Image",
        validators=[
            FileRequired(),
            FileAllowed(["jpg", "png"], "Only JPG and PNG images are allowed"),
        ],
    )
    tags = StringField(
        "Tags",
        validators=[
            DataRequired(),
            Regexp(
                r"" + config.get("PROJECTS", "tags_regex"),
                message=config.get("PROJECTS", "tags_regex_msg"),
            ),
        ],
    )


class AuthRequestForm(FlaskForm):
    email = StringField(
        "Email",
        validators=[
            DataRequired(),
            Email(message="Invalid email address format"),
            Regexp(
                r"" + config.get("PROJECTS", "auth_regex_username"),
                message=config.get("PROJECTS", "auth_regex_msg"),
            ),
        ],
    )


class PasswordResetRequestForm(FlaskForm):
    email = StringField(
        "Email",
        validators=[
            DataRequired(),
            Email(message="Invalid email address format"),
            Regexp(
                r"" + config.get("PROJECTS", "auth_regex_username"),
                message=config.get("PROJECTS", "auth_regex_msg"),
            ),
        ],
    )


class PasswordResetForm(FlaskForm):
    password = PasswordField(
        "Password",
        validators=[
            DataRequired(),
            Length(min=8),
            EqualTo("confirm", message="Passwords must match"),
        ],
    )
    confirm = PasswordField("Repeat Password")


class LoginForm(FlaskForm):
    email = StringField(
        "Email",
        validators=[
            DataRequired(),
            Email(message="Invalid email address format"),
            Regexp(
                r"" + config.get("PROJECTS", "auth_regex_username"),
                message=config.get("PROJECTS", "auth_regex_msg"),
            ),
        ],
    )
    password = PasswordField("Password", validators=[DataRequired()])



import os
import unittest
import io
import json
import datetime
from bcrypt import hashpw, gensalt
from flask import g
from projects import app
from projects.models import Project, User


libsum = (
    "Lorem ipsum dolor sit amet, consectetur adipiscing elit."
    " Maecenas semper tortor justo. Vivamus pretium quis leo quis "
    "posuere. In tristique orci ac orci laoreet, in imperdiet elit "
    "semper. Nam id eros gravida, fringilla nisl a, laoreet nibh. "
    "Mauris viverra, risus posuere pellentesque blandit, "
    "ipsum augue dignissim mi, vitae consectetur magna libero eu purus."
    " Interdum et malesuada fames ac ante ipsum primis in faucibus."
    " Ut sagittis bibendum nulla, et malesuada erat ultricies vel."
    " Duis vitae nisi augue. In blandit, purus ut consectetur suscipit,"
    " erat quam euismod nibh, gravida vestibulum purus velit nec libero"
    ". Nullam blandit auctor dolor in pretium. Nulla posuere magna non"
    " neque malesuada, tempor interdum metus aliquam. Nam tincidunt"
    " pellentesque congue. Duis auctor, tellus sit amet vehicula"
    " tempus, nunc purus ultrices lacus, nec convallis turpis nunc id"
    " purus. Cras aliquet dapibus convallis. Fusce fermentum velit enim"
    ", eu cursus enim fermentum at."
)


class Test_ProjectsTestCase(unittest.TestCase):
    def setUp(self):
        app.config["TESTING"] = True
        app.config["DEBUG"] = True
        folders = {}
        folders["DATA_FOLDER"] = app.config["DATA_FOLDER"] = os.path.join(
            os.getcwd(), "tests/data"
        )

        folders["UPLOAD_FOLDER"] = app.config["UPLOAD_FOLDER"] = os.path.join(
            os.getcwd(), "tests/images"
        )
        app.config["WTF_CSRF_ENABLED"] = True
        # Create required folders for the application if they don't exist
        for _, folder in folders.items():
            try:
                os.makedirs(folder)
                print("Created: " + folder)
            except FileExistsError:
                pass

        # Override default DB setting ->use a testing db instead of the default
        app.config["DB"] = os.path.join(app.config["DATA_FOLDER"], "fair_test")
        self.username = "test@test.com"
        self.password = "test"
        user = User.get_with_first("email", self.username)
        hashed_pw = hashpw(bytes(self.password, "utf-8"), gensalt())

        if user is None:
            user = User(
                email=self.username,
                password=hashed_pw,
                projects=[],
                is_active=True,
                is_authenticated=True,
                is_anonymous=False,
                confirmed_on=datetime.datetime.now(),
            )
            user.save()
        self.user = user

        self.client = app.test_client()
        # Setup valid token
        self.csrf_token = None
        with self.client as client:
            resp = client.get("/index")
            assert resp.status_code == 200
            self.csrf_token = g.csrf_token

    def tearDown(self):
        # Clean up
        Project.clear()
        self.assertTrue(len(Project.get_all()) == 0)

    def login(self, client):
        token_data = {"csrf_token": self.csrf_token}
        login_data = {"email": self.username, "password": self.password}
        login_data.update(token_data)
        login_resp = client.post("/login", data=login_data, follow_redirects=True)
        self.assertEqual(login_resp.status_code, 200)

    def register_project(self, csrf_token, **kwargs):
        data = dict(csrf_token=csrf_token)
        data.update(kwargs)
        return self.client.post("/create_project", data=data, follow_redirects=True,)

    def tag_post_query(self, tag):
        return self.client.post(
            "/search", data=dict(tag=tag, csrf_token=self.csrf_token)
        )

    def tag_get_query(self, tag):
        return self.client.get("/tag?tag=" + tag)

    def test_register_data_render(self):
        # Auth check
        with self.client as client:
            create_resp = client.get("/create_project", follow_redirects=True)
            test_response = "Please log in to access this page."
            self.assertIn(bytes(test_response, encoding="utf8"), create_resp.data)
            # setup csrf_token
            token_data = {"csrf_token": self.csrf_token}
            self.login(client)
            create_resp = client.get(
                "create_project", data=token_data, follow_redirects=True
            )
            test_response = "Register a Project"
            self.assertIn(bytes(test_response, encoding="utf8"), create_resp.data)

    # TODO -> improve image validation by comparing hash of initial
    # and uploaded image, not just the returned page
    def test_web_upload_project(self):
        # Register project
        image_name = "robo_test_image.png"
        image_path = os.path.join(os.path.dirname(__file__), "images", image_name)
        with open(image_path, "rb") as img:
            image = img.read()

        with self.client as client:
            self.login(client)
            rv = self.register_project(
                name="My Awesome project",
                description=libsum,
                tags="Robots,Evolution",
                image=(io.BytesIO(image), image_name),
                csrf_token=self.csrf_token,
            )

            test_response = (
                "Your submission has been received, " "your metadata can be found at:"
            )
            self.assertIn(bytes(test_response, encoding="utf8"), rv.data)

    def test_web_upload_project_ten(self):
        # upload 10
        iterations = 10
        with self.client as client:
            self.login(client)

            image_name = "roborobo_simulation.png"
            image_path = os.path.join(os.path.dirname(__file__), "images", image_name)
            with open(image_path, "rb") as img:
                image = img.read()

            for _ in range(iterations):
                rv = self.register_project(
                    name="My Awesome project",
                    description=libsum,
                    tags="AnotherTag",
                    image=(io.BytesIO(image), image_name),
                    csrf_token=self.csrf_token,
                )

                test_response = (
                    "Your submission has been received, "
                    "your metadata can be found at:"
                )
                self.assertIn(bytes(test_response, encoding="utf8"), rv.data)

    def test_project_save_with_orcid(self):
        with self.client as client:
            # Authenticate
            self.login(client)

            image_name = "robo_test_image.png"
            image_path = os.path.join(os.path.dirname(__file__), "images", image_name)
            with open(image_path, "rb") as img:
                image = img.read()

            rv = self.register_project(
                name="My Awesome project",
                description=libsum,
                tags="Robots,Evolution,Individual Learning",
                image=(io.BytesIO(image), image_name),
                csrf_token=self.csrf_token,
            )

            test_response = (
                "Your submission has been received, " "your metadata can be found at:"
            )
            self.assertIn(bytes(test_response, encoding="utf8"), rv.data)

            # Clean up
            Project.clear()
            self.assertTrue(len(Project.get_all()) == 0)

    def test_project_save(self):
        project = Project(
            name="nasiodas",
            description="sdifosdfios",
            image="/Usdufsi/sdfs.png",
            tags="sdfo,sdfo,sdf",
        )
        project_id = project.save()
        new_project = Project.get(project_id)
        [
            self.assertEqual(project.__dict__[key], new_project.__dict__[key])
            for key in project.__dict__.keys()
        ]

    def test_project_update(self):
        # Pre cleanup
        self.user.projects.clear()
        self.user.save()
        # Save a new project
        project = Project(
            name="nasiodas",
            description="sdifosdfios",
            image=os.path.join("Usdufsi", "sdfs.png"),
            tags="sdfo,sdfo,sdf",
        )
        project_id = project.save()
        new_project = Project.get(project_id)
        [
            self.assertEqual(project.__dict__[key], new_project.__dict__[key])
            for key in project.__dict__.keys()
        ]

        # Authenticate
        with self.client as client:
            self.login(client)
            # Update project without being the correct owner
            new_data = {
                "name": "My Awesome project",
                "description": "nice_description",
                "tags": "Robots,Evolution,Individual Learning",
                "image": os.path.join("sdfsfs", "sdfsf.png"),
                "csrf_token": self.csrf_token,
            }
            rv = client.post(
                "/update/" + project_id, data=new_data, follow_redirects=True
            )
            test_response = "Your trying to update an entity that"
            self.assertIn(bytes(test_response, encoding="utf8"), rv.data)
            self.user.projects.append(project_id)
            self.user.save()
            rv = client.post(
                "/update/" + project_id, data=new_data, follow_redirects=True
            )
            test_response = "Update Success, your data can be found at: "
            self.assertIn(bytes(test_response, encoding="utf8"), rv.data)

    def test_project_getall_clear(self):
        Project.clear()
        self.assertTrue(len(Project.get_all()) == 0)
        num_project = 10
        for _ in range(num_project):
            project = Project(
                name="nasiodas",
                description="sdifosdfios",
                image=os.path.join("Usdufsi", "sdfs.png"),
                tags="sdfo,sdfo,sdf",
            )
            project.save()
        self.assertEqual(len(Project.get_all()), num_project)

    # Tests whether a list of submitted projects with tags are
    # properly returned when search for with the post request (Search Bar)
    def test_tag_post_search(self):
        Project.clear()
        self.assertTrue(len(Project.get_all()) == 0)

        first_tag = "eScience:bohrium:tests"
        second_tag = "eScience:bohrium:benchmark"
        num_project = 10
        for num in range(num_project):
            if num % 2 == 0:
                project = Project(
                    name="nasiodas",
                    description="sdifosdfios",
                    image=os.path.join("Usdufsi", "sdfs.png"),
                    tags=first_tag,
                )
            else:
                project = Project(
                    name="nasiodas",
                    description="sdifosdfios",
                    image=os.path.join("Usdufsi", "sdfs.png"),
                    tags=second_tag,
                )
            project.save()

        # Test Post Search -> json response evaluated
        first_query = json.loads(
            self.tag_post_query(tag=first_tag).get_data(as_text=True)
        )
        for project in first_query["data"]:
            self.assertTrue(project["tags"] == first_tag)

        second_query = json.loads(
            self.tag_post_query(tag=second_tag).get_data(as_text=True)
        )
        for project in second_query["data"]:
            # TODO -> test URL GET search query
            self.assertTrue(project["tags"] == second_tag)


if __name__ == "__main__":
    unittest.main()



# Common configuration
import os
import configparser

# Load config.ini
res_path = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "..", "res", "config.ini"
)
config = configparser.SafeConfigParser()
config.read(res_path)

# common static folder path
config.get("BASE", "static_folder")
config.get("BASE", "upload_folder")
config.get("BASE", "db_lock")



import os
import datetime
from flask import render_template, request, flash, redirect, url_for, jsonify
from flask_login import login_user, logout_user, login_required, current_user
from flask_mail import Message
from werkzeug.datastructures import CombinedMultiDict
from werkzeug.utils import secure_filename
from bcrypt import hashpw, gensalt
from projects import mail, project_manager, projects_blueprint, app
from projects.conf import config
from projects.models import Project, User
from projects.forms import (
    AuthRequestForm,
    PasswordResetRequestForm,
    LoginForm,
    PasswordResetForm,
    FileRequired,
)
from projects.helpers import unique_name_encoding, unique_name_decode
from projects_base.base.forms import TagsSearchForm
from projects.helpers import generate_confirmation_token, confirm_token


# Routes
@projects_blueprint.route("/")
@projects_blueprint.route("/index", methods=["GET"])
def projects():
    form = TagsSearchForm()
    entities = Project.get_all()
    tags = Project.get_top_with("tags", num=10)
    return render_template(
        "projects/projects.html",
        title=config.get("PROJECTS", "title"),
        grid_header="{} {}".format(config.get("PROJECTS", "title"), "Projects"),
        tags=list(tags.keys()),
        objects=entities,
        form=form,
    )


@projects_blueprint.route("/my_projects", methods=["GET"])
@login_required
def my_projects():
    form = TagsSearchForm()
    entities = [
        project for project in Project.get_all() if project._id in current_user.projects
    ]

    return render_template("projects/projects.html", objects=entities, form=form)


@projects_blueprint.route("/show/<object_id>", methods=["GET"])
def show(object_id):
    form_class = project_manager.get_form_class()
    form = form_class()
    entity = Project.get(object_id)
    if entity is None:
        flash("That project doesn't exist", "danger")
        return redirect(url_for(".projects"))

    owner = False
    if current_user.is_authenticated and object_id in current_user.projects:
        owner = True
        for attr in entity.__dict__:
            if attr != "_id" and attr != "_type":
                form[attr].data = entity.__dict__[attr]
            # Workaround for image upload required, set the label
            # to the currently used image and disable it as a required field
            if attr == "image":
                form[attr].label.text = "Stored image is: " + unique_name_decode(
                    entity.__dict__[attr]
                )
                form[attr].flags = None
    else:
        # Translate area keys to values
        if hasattr(entity, "area"):
            entity.area = [
                area[1] for area in form.area.choices if area[0] in entity.area
            ]
    return render_template(
        "projects/project.html", object=entity, owner=owner, form=form
    )


@projects_blueprint.route("/create_project", methods=["GET", "POST"])
@login_required
def create():
    form_class = project_manager.get_form_class()
    form = form_class(CombinedMultiDict((request.files, request.form)))

    if form.validate_on_submit():
        f = form.image.data
        # Make sure the saved image is filename is unique
        filename = secure_filename(unique_name_encoding(f.filename))
        f.save(os.path.join(config.get("PROJECTS", "upload_folder"), filename))
        # Remove special fields
        if form.__contains__("csrf_token"):
            form._fields.pop("csrf_token")
        form._fields.pop("image")
        # Save new instance
        new_instance = {
            key: field.data
            for key, field in form.__dict__.items()
            if hasattr(field, "data")
        }

        new_instance["image"] = filename
        entity = Project(**new_instance)
        entity_id = entity.save()
        # Update user with new instance
        current_user.projects.append(entity_id)
        current_user.save()
        url = url_for("projects.show", object_id=entity_id, _external=True)
        flash(
            "Your submission has been received,"
            " your metadata can be found at: " + url,
            "success",
        )
        return redirect(url)
    return render_template("projects/create_project.html", form=form)


@projects_blueprint.route("/update/<object_id>", methods=["POST"])
@login_required
def update(object_id):
    entity = Project.get(object_id)
    if entity is None:
        flash("That entity dosen't exist", "danger")
        return redirect(url_for("projects.projects"))

    if object_id not in current_user.projects:
        flash("Your trying to update an entity that's not yours", "danger")
        return redirect(url_for("projects.projects"))

    form_class = project_manager.get_form_class()
    form = form_class(CombinedMultiDict((request.files, request.form)))
    # Strip image upload validation on upload (Optional)
    form.image.validators = [
        validator
        for validator in form.image.validators
        if type(validator) is not FileRequired
    ]

    if form.validate_on_submit():
        # Only save the image if a new was submitted, else keep the old name
        f = form.image.data
        if f and hasattr(f, "filename") and f.filename != "":
            filename = secure_filename(unique_name_encoding(f.filename))
            f.save(os.path.join(config.get("PROJECTS", "upload_folder"), filename))
            # Remove old
            os.remove(
                os.path.join(config.get("PROJECTS", "upload_folder"), entity.image)
            )
        else:
            filename = entity.image

        # Update every attribute except _id, _type, and image
        disabled_updates = ["_id", "_type", "image"]
        for attr in entity.__dict__:
            if attr not in disabled_updates:
                entity.__dict__[attr] = form[attr].data
        entity.__dict__["image"] = filename
        entity_id = entity.save()

        url = url_for("projects.show", object_id=entity_id, _external=True)
        flash("Update Success, your data can be found at: " + url, "success")
        return redirect(url)
    form.image.flags = None
    return render_template("projects/project.html", object=entity, form=form)


@projects_blueprint.route("/delete/<object_id>", methods=["POST"])
@login_required
def delete(object_id):
    entity = Project.get(object_id)
    if entity is None:
        flash("That entity dosen't exist", "danger")
        return redirect(url_for("projects.projects"))
    if object_id in current_user.projects:
        Project.remove(object_id)
        current_user.projects.remove(object_id)
        os.remove(os.path.join(config.get("PROJECTS", "upload_folder"), entity.image))
        current_user.save()
        flash("Entity: " + entity.name + " has been deleted", "success")
    else:
        flash("Your trying to delete an entity you don't own", "danger")
    return redirect(url_for("projects.projects"))


# Sends approval emails to every app.config['ADMINS_EMAIL']
@projects_blueprint.route("/request_auth", methods=["POST"])
def request_auth():
    form = AuthRequestForm(request.form)
    if form.validate_on_submit():
        # Send confirmation token
        user = User.get_with_first("email", form.email.data)
        if user is None:
            data = form.data
            # Remove csrf_token
            del data["csrf_token"]
            subject = "{} requests {} access".format(
                form.email.data, config.get("PROJECTS", "title")
            )
            token = generate_confirmation_token(data=form.data)
            confirm_url = url_for("projects.approve_auth", token=token, _external=True)
            html = render_template(
                "projects/email/activate_user.html",
                email=form.data,
                confirm_url=confirm_url,
            )
            msg = Message(
                subject=subject,
                html=html,
                recipients=[app.config["ADMINS_EMAIL"]],
                sender=app.config["MAIL_USERNAME"],
            )
            try:
                mail.send(msg)
            except TimeoutError:
                return jsonify(
                    data={
                        "danger": "Timed out before request could be sent"
                        " to an admin for approval"
                    }
                )
            return jsonify(
                data={
                    "success": "Request successfully submitted"
                    ", awaiting admin approval"
                }
            )
        else:
            response = jsonify(
                data={"danger": "That email has already been granted access"}
            )
            response.status_code = 400
            return response
    response = jsonify(
        data={
            "danger": ", ".join(
                [
                    "{} - {}".format(attr, r_msg)
                    for attr, errors in form.errors.items()
                    for r_msg in errors
                ]
            )
        }
    )
    response.status_code = 400
    return response


@projects_blueprint.route("/request_password_reset", methods=["POST"])
def request_password_reset():
    form = PasswordResetRequestForm(request.form)
    if form.validate_on_submit():
        user = User.get_with_first("email", form.email.data)
        if user is None:
            response = jsonify(data={"danger": "That user does not exist"})
            response.status_code = 400
            return response
        else:
            email = user.email
            token = generate_confirmation_token(data=email)
            reset_url = url_for("projects.reset_password", token=token, _external=True)
            html = render_template(
                "projects/email/reset_password.html",
                email=email,
                reset_password_url=reset_url,
            )
            msg = Message(
                subject="{} Reset Password".format(config.get("PROJECTS", "title")),
                html=html,
                recipients=[email],
                sender=app.config["MAIL_USERNAME"],
            )
            mail.send(msg)
            return jsonify(
                data={
                    "success": "A password reset link has been sent to {}".format(email)
                }
            )
    response = jsonify(
        data={
            "danger": ", ".join(
                [
                    "{} - {}".format(attr, r_msg)
                    for attr, errors in form.errors.items()
                    for r_msg in errors
                ]
            )
        }
    )
    response.status_code = 400
    return response


# Accepts approval from admin's
@projects_blueprint.route("/approve_auth/<token>")
def approve_auth(token):
    data = confirm_token(token)
    if data is False:
        flash("Confirmation failed, either it is invalid or expired.", "danger")
        return redirect(url_for("projects.projects"))

    if "email" not in data:
        flash("Confirmation failed, required email is not present", "danger")
        return redirect(url_for("projects.projects"))

    user = User.get_with_first("email", data["email"])
    if user is not None:
        flash("That email has already been registered")
        return redirect(url_for("projects.projects"))
    else:
        # Setup
        user = User(
            email=data["email"],
            password=hashpw(os.urandom(24), gensalt()),
            projects=[],
            is_active=False,
            is_authenticated=False,
            is_anonymous=False,
            confirmed_on=datetime.datetime.now(),
        )
        user.save()

        token = generate_confirmation_token(data=data["email"])
        reset_url = url_for("projects.reset_password", token=token, _external=True)
        html = render_template(
            "projects/email/reset_password.html",
            email=data["email"],
            reset_password_url=reset_url,
        )
        msg = Message(
            subject="{} Projects Account approval".format(
                config.get("PROJECTS", "title")
            ),
            html=html,
            recipients=[data["email"]],
            sender=app.config["MAIL_USERNAME"],
        )
        mail.send(msg)
        flash(
            "The account {} has been approved and created".format(data["email"]),
            "success",
        )
    return redirect(url_for("projects.projects"))


@projects_blueprint.route("/reset_password/<token>", methods=["POST", "GET"])
def reset_password(token):
    email = confirm_token(token)
    if email is False:
        flash(
            "Attempted password reset failed,"
            " the request is either invalid or expired",
            "danger",
        )
        return redirect(url_for("projects.login"))

    form = PasswordResetForm(request.form)
    if form.validate_on_submit():
        user = User.get_with_first("email", email)
        user.is_active = True
        user.is_authenticated = True
        user.is_anonymous = False
        user.email = email
        user.password = hashpw(bytes(form.password.data, "utf-8"), gensalt())
        user.save()
        flash("Your password has now been updated", "success")
        return redirect(url_for("projects.projects"))
    return render_template("projects/reset_password_form.html", form=form, email=email)


# @app.route('/login', methods=['GET', 'POST'])
@projects_blueprint.route("/login", methods=["GET", "POST"])
def login():
    form = LoginForm(request.form)
    if form.validate_on_submit():
        valid_user = User.valid_user(form.email.data, form.password.data)
        if valid_user is not None:
            flash("Logged in successfully.", "success")
            login_user(valid_user)
            return redirect(url_for("projects.projects"))
        else:
            flash("Invalid Credentials", "danger")
    return render_template("projects/login.html", form=form)


@projects_blueprint.route("/logout")
@login_required
def logout():
    logout_user()
    return redirect(url_for("projects.projects"))


@projects_blueprint.route("/tag/<tag>", methods=["GET"])
def tag_search(tag):
    form = TagsSearchForm(data={"tag": tag}, meta={"csrf": False})
    entities = {}
    tags = Project.get_top_with("tags")
    if form.validate():
        entities = Project.get_with_search("tags", form.tag.data)
    return render_template(
        "projects/projects.html", tags=list(tags.keys()), objects=entities, form=form
    )


# TODO -> refactor with fair search forms in common views instead.
@projects_blueprint.route("/search", methods=["GET"])
def tag_external_search():
    form = TagsSearchForm(request.args, meta={"csrf": False})
    entities = {}
    # The return form should contain a csrf_token
    return_form = TagsSearchForm()
    return_form.tag = form.tag
    if form.validate():
        entities = Project.get_with_search("tags", form.tag.data)
        return render_template(
            "projects/projects.html", objects=entities, form=return_form
        )
    # pass on errors
    return_form._errors = form.errors
    return_form._fields["tag"] = form._fields["tag"]
    return render_template("projects/projects.html", objects=entities, form=return_form)


# TODO -> refactor with fair search forms in common views instead.
@projects_blueprint.route("/search", methods=["POST"])
def tag_native_search():
    form = TagsSearchForm(request.form)
    if form.validate_on_submit():
        result = {}
        tag = request.form["tag"]
        entities = Project.get_with_search("tags", tag)
        if len(entities) > 0:
            result = [entity.serialize() for entity in entities]
        return jsonify(data=result)

    response = jsonify(
        data={
            "danger": ", ".join(
                [msg for attr, errors in form.errors.items() for msg in errors]
            )
        }
    )
    response.status_code = 400
    return response



from flask_wtf import FlaskForm
from wtforms import StringField
from wtforms.validators import Optional, Regexp


class TagsSearchForm(FlaskForm):
    tag = StringField(
        "Tags Search",
        validators=[
            Optional(),
            Regexp(
                r"^[\w,:_\-* ]*$",
                message="Allowed tag characters "
                "include letters spaces and"
                ", : _ - *",
            ),
        ],
    )



from flask_login import current_user
from flask_nav.elements import Navbar, View
from projects.conf import config
from projects import nav


@nav.navigation()
def nav_bar():
    navbar = list(
        Navbar(
            View("{}".format(config.get("PROJECTS", "title")), ".projects"),
            View("Projects", ".projects"),
        ).items
    )
    if current_user.is_authenticated:
        navbar.extend(
            [
                View("My Projects", "projects.my_projects"),
                View("Create Project", "projects.create"),
                View("Logout", "projects.logout"),
            ]
        )
    else:
        navbar.extend(
            [View("Login", "projects.login"),]
        )

    return Navbar("{}".format(config.get("PROJECTS", "title")), *navbar)



import os
from flask import Flask, Blueprint
from flask_wtf.csrf import CSRFProtect
from flask_bootstrap import Bootstrap
from flask_login import LoginManager
from flask_mail import Mail
from flask_nav import Nav
from projects_base.base import base_blueprint
from projects.models import User
from projects.forms import ProjectFormManager, DefaultProjectForm
from projects.conf import config

app = Flask(__name__)
app.register_blueprint(base_blueprint)
projects_blueprint = Blueprint(
    "projects",
    __name__,
    static_folder="static",
    static_url_path="/projects/static",
    template_folder="templates",
)

csrf = CSRFProtect(app)
app.secret_key = os.urandom(24)
Bootstrap(app)

# Login Manager
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = "projects.login"

project_manager = ProjectFormManager()

# Onetime authentication reset token salt
app.config["ONETIME_TOKEN_SALT"] = os.urandom(24)

# TODO -> fix hack at some point
# Application ADMINS_EMAIL
if "ADMINS_EMAIL" in os.environ:
    app.config["ADMINS_EMAIL"] = os.environ["ADMINS_EMAIL"].split(",")
else:
    app.config["ADMINS_EMAIL"] = config.get("MAIL", "admins")

# Email application server
if "MAIL_SERVER" in os.environ:
    app.config["MAIL_SERVER"] = os.environ["MAIL_SERVER"]
else:
    app.config["MAIL_SERVER"] = config.get("MAIL", "server")

if "MAIL_PORT" in os.environ:
    app.config["MAIL_PORT"] = os.environ["MAIL_PORT"]
else:
    app.config["MAIL_PORT"] = config.get("MAIL", "port")

if "MAIL_USE_TLS" in os.environ:
    app.config["MAIL_USE_TLS"] = bool(os.environ["MAIL_USE_TLS"])
else:
    # NOTE!, remember that if any value is set for use_tls it is set to True
    app.config["MAIL_USE_TLS"] = bool(config.get("MAIL", "use_tls"))

if "MAIL_USE_SSL" in os.environ:
    app.config["MAIL_USE_SSL"] = (
        False if os.environ["MAIL_USE_SSL"] == "False" else True
    )
else:
    # NOTE!, remember that if any value is set for use_ssl it is set to True
    app.config["MAIL_USE_SSL"] = bool(config.get("MAIL", "use_ssl"))

if "MAIL_USERNAME" in os.environ:
    app.config["MAIL_USERNAME"] = os.environ["MAIL_USERNAME"]
else:
    app.config["MAIL_USERNAME"] = config.get("MAIL", "username")

if "MAIL_PASSWORD" in os.environ:
    app.config["MAIL_PASSWORD"] = os.environ["MAIL_PASSWORD"]
else:
    app.config["MAIL_PASSWORD"] = config.get("MAIL", "password")

# Connect mail
mail = Mail(app)

# Setup navbar
nav = Nav()
nav.init_app(app)
import projects.nav
import projects.views

app.register_blueprint(projects_blueprint)



from flask import Blueprint
from projects_base.base.conf import config

base_blueprint = Blueprint(
    "projects_base",
    __name__,
    static_folder="static",
    static_url_path="/base/static",
    template_folder="templates",
)
import projects_base.base.views



import os
from projects_base.base import base_blueprint
from projects_base.base.conf import config
from flask import send_from_directory


@base_blueprint.route("/images/<path:filename>", methods=["GET"])
def img_render(filename):
    return send_from_directory(
        os.path.abspath(config.get("BASE", "upload_folder")), filename=filename
    )



# http://flask.pocoo.org/docs/0.12/patterns/distribute/
import os
from setuptools import find_packages
from distutils.core import setup

here = os.path.dirname(__file__)
long_description = open("README.rst").read()

version_ns = {}
with open(os.path.join(here, "version.py")) as f:
    exec(f.read(), {}, version_ns)

setup(
    name="projects-site",
    version=version_ns["__version__"],
    long_description=long_description,
    description="""
                A website template for hosting project
                metadata information via flask
                """,
    url="https://github.com/rasmunk/projects-site",
    author="Rasmus Munk",
    author_email="rasmus.munk@nbi.ku.dk",
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    keywords=["Website", "Flask", "MetaData"],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "Flask",
        "flask-bootstrap",
        "flask-nav",
        "flask_wtf",
        "wtforms",
        "requests",
        "wtforms-components",
        "flask_login",
        "flask_mail",
        "bcrypt",
        "configparser",
    ],
    setup_requires=["pytest-runner",],
    tests_require=["pytest",],
)
